﻿Shader "NPR/ContoursDoG"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always
		Pass
	{
		CGPROGRAM
#pragma vertex vert
#pragma fragment frag

#include "UnityCG.cginc"

		struct appdata
		{
			float4 vertex : POSITION;
			float2 uv : TEXCOORD0;
		};

		struct v2f
		{
			float2 uv : TEXCOORD0;
			float4 vertex : SV_POSITION;
		};

		v2f vert(appdata v)
		{
			v2f o;
			o.vertex = UnityObjectToClipPos(v.vertex);
			o.uv = v.uv;
			return o;
		}

		sampler2D _MainTex;

		float4 frag(v2f i) : SV_Target
		{
			float L = dot(tex2D(_MainTex, i.uv).rgb, float3(0.2126, 0.7152, 0.0722));
		return float4(L,L,L,1);
		}
			ENDCG
		}
		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			#include "UnityCG.cginc"

				struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert(appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}

			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			float sigma;

			float Gauss(float d, float sigma)
			{
				return 1.0 / (sigma*sqrt(2.0*3.14)) * exp((-d*d) / (2.0*sigma*sigma));
			}

			float4 frag(v2f i) : SV_Target
			{
				float3 SUM = float3(0,0,0);
				float SUMW = 0;
				if (sigma == 0)
					SUM = tex2D(_MainTex, i.uv).rgb;
				else
				{
					for (int iy = -5; iy <= 5; iy++)
						for (int ix = -5; ix <= 5; ix++)
						{
							float2 uv = i.uv + _MainTex_TexelSize.xy * float2(ix, iy);
							float3 col = tex2D(_MainTex, uv).rgb;
							float w = Gauss(length((i.uv - uv) / _MainTex_TexelSize.xy), sigma);
							SUMW += w;
							SUM += col*w;
						}
					SUM /= SUMW;
				}

				return float4(SUM, 1);
			}
			ENDCG
		}
		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			sampler2D Gauss1Tex;
			sampler2D Gauss2Tex;
			float DoGThreshold;
			float diffThreshold;
			float DoGFalloff;
			float diffFalloff;

			float4 frag (v2f i) : SV_Target
			{
				float G1 = tex2D(Gauss1Tex, i.uv).r;
				float G2 = tex2D(Gauss2Tex, i.uv).r;
				float DoG = abs(G1 - G2);
				float edgeValue = 1.0 - smoothstep(DoGThreshold, DoGThreshold + DoGFalloff, DoG);
				float gradX = tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2(1, 0)).r
					- tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2(-1, 0)).r;
				float gradY = tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2(0, 1)).r
					- tex2D(_MainTex, i.uv + _MainTex_TexelSize*float2(0, -1)).r;
				float gradMag = length(float2(gradX, gradY));
				edgeValue *= smoothstep(max(0, diffThreshold - diffFalloff), diffThreshold + diffFalloff, gradMag);
				edgeValue = 1.0 - edgeValue;
				float4 imageColor = tex2D(_MainTex, i.uv);
				imageColor.rgb *= edgeValue;
				//return float4(edgeValue, edgeValue, edgeValue, 1);
				return imageColor;				
			}
			ENDCG
		}
	}
}
