﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class ContoursLaplace : PostEffectsBase {
    [Range(0, 0.2f)]
    public float LaplaceThreshold = 0.01f;
    [Range(0, 0.5f)]
    public float LaplaceFalloff = 0.01f;
    [Range(0, 1.0f)]
    public float gradientThreshold = 0.05f;
    [Range(0, 0.5f)]
    public float gradientFalloff= 0.01f;
    [Range(0, 5.0f)]
    public float sigma = 1.0f;
    void Start()
    {
        shader = Shader.Find("NPR/ContoursLaplace");
        CreateMaterial();
    }

    void OnRenderImage(RenderTexture source, RenderTexture destination)
    {
        if (CreateMaterial())
        {
            RenderTexture tmp = RenderTexture.GetTemporary(source.width, source.height, 0, RenderTextureFormat.DefaultHDR);
            Graphics.Blit(source, tmp, material, 0);
            material.SetFloat("LaplaceThreshold", LaplaceThreshold); 
            material.SetFloat("gradientThreshold", gradientThreshold);
            material.SetFloat("LaplaceFalloff", LaplaceFalloff);
            material.SetFloat("gradientFalloff", gradientFalloff); 
            material.SetFloat("sigma", sigma);
            material.SetTexture("ColorTex", source);
            Graphics.Blit(tmp, destination, material, 1);
            RenderTexture.ReleaseTemporary(tmp);
        }
        else
        {
            Graphics.Blit(source, destination);
        }
    }
}
