﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class BilateralFilter : PostEffectsBase
{
    [Range(0, 5.0f)]
    public float sigmaDistance = 1.0f;
    [Range(0, 2.0f)]
    public float sigmaValue = 1.0f;
    public int iterations = 1;

    void Start () {
        shader = Shader.Find("NPR/BilateralFilter");
        CreateMaterial();
    }

    void OnRenderImage(RenderTexture source, RenderTexture destination)
    {
        if (CreateMaterial())
        {
            RenderTexture tmp = RenderTexture.GetTemporary(source.width, source.height, 0, RenderTextureFormat.DefaultHDR);
            RenderTexture rt1 = source;
            RenderTexture rt2 = tmp;
            for (int i = 0; i < iterations; ++i)
            {
                material.SetFloat("sigmaDistance", sigmaDistance);
                material.SetFloat("sigmaValue", sigmaValue);
                material.SetInt("filterDir", 0);
                Graphics.Blit(rt1, rt2, material);
                material.SetInt("filterDir", 1);
                if (i == iterations - 1)
                    rt1 = destination;
                Graphics.Blit(rt2, rt1, material);
            }
            tmp.Release();
        }
        else
        {
            Graphics.Blit(source, destination);
        }
    }
}
