﻿Shader "NPR/KuwaharaFilter"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			int filterSize;

			float4 frag (v2f I) : SV_Target
			{
				int N = filterSize;
			    //Kuwahara filter divides the filter window to four equal parts
			    //PcountInv is the inv number of pixels in one window part
				float PcountInv = 1.0 / float((N + 1)*(N + 1));
				//First window region
				float3 E1 = float3(0, 0, 0); //expected value of region
				float3 E1_2 = float3(0, 0, 0);//expected squared value of region
				for (int i = -N; i <= 0; i++)
					for (int j = -N; j <= 0; j++)
				{
				    float3 P = tex2D(_MainTex, I.uv + _MainTex_TexelSize.xy*float2(i, j)).rgb;
					E1 += P;
					E1_2 += P*P;
				}
				E1 *= PcountInv;
				float V1 = length(PcountInv*E1_2 - E1*E1);//variance of region
				//Second window region
				float3 E2 = float3(0, 0, 0);
				float3 E2_2 = float3(0, 0, 0);
				for (int i = -N; i <= 0; i++)
				    for (int j = 0; j <= N; j++)
				{
					float3 P = tex2D(_MainTex, I.uv + _MainTex_TexelSize.xy*float2(i, j)).rgb;
					E2 += P;
					E2_2 += P*P;
				}
				E2 *= PcountInv;
				float V2 = length(PcountInv*E2_2 - E2*E2);
				//Third window region
				float3 E3 = float3(0, 0, 0);
				float3 E3_2 = float3(0, 0, 0);
				for (int i = 0; i <= N; i++)
				    for (int j = 0; j <= N; j++)
				{
					float3 P = tex2D(_MainTex, I.uv + _MainTex_TexelSize.xy*float2(i, j)).rgb;
					E3 += P;
					E3_2 += P*P;
				}
				E3 *= PcountInv;
				float V3 = length(PcountInv*E3_2 - E3*E3);
				//Fourth window region
				float3 E4 = float3(0, 0, 0);
				float3 E4_2 = float3(0, 0, 0);
				for (int i = 0; i <= N; i++)
					for (int j = -N; j <= 0; j++)
				{
					float3 P = tex2D(_MainTex, I.uv + _MainTex_TexelSize.xy*float2(i, j)).rgb;
					E4 += P;
					E4_2 += P*P;
				}
				E4 *= PcountInv;
				float V4 = length(PcountInv*E4_2 - E4*E4);
				//Find the region with least variance
				float3 minE = E1;
				float minV = V1;				
				if (V2 < minV) { minE = E2; minV = V2; }
				if (V3 < minV) { minE = E3; minV = V3; }
				if (V4 < minV) { minE = E4; minV = V4; }
				//Use the excepted value of the region with least variance
			    return float4(minE, 1);
			}
			ENDCG
		}
	}
}
