﻿Shader "NPR/SNNFilter"
{
	Properties
	{
		_MainTex ("Texture", 2D) = "white" {}
	}
	SubShader
	{
		// No culling or depth
		Cull Off ZWrite Off ZTest Always

		Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			
			#include "UnityCG.cginc"

			struct appdata
			{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
				return o;
			}
			
			sampler2D _MainTex;
			float4 _MainTex_TexelSize;
			int filterSize;

			float4 frag (v2f i) : SV_Target
			{
				int N = filterSize;
				//Inverse of the number of pixels pairs we are going to visit
				//It will be needed for taking the average
				float PcountInv = 1.0 / float((2 * N + 1)*N + N);
				//Read the center pixel value in the filter window
				float3 PCenter = tex2D(_MainTex, i.uv).rgb;
				float3 AVG = float3(0, 0, 0);
				//SNN filter reads pair of pixels in filter window
				//The pairs are pixels mirrored to the filter origin
				//Thus we only have to loop the half of the filter window
				for (int iy = -N; iy <= 0; iy++)
					for (int ix = -N; ix <= N; ix++)
				{
					if (iy == 0 && ix == 0)//if we reached the half of the filter window
					{ ix = N + 1; iy = 1; break; }//quit the double loop
					//sample the pixel and its mirrored pair
					float3 P1 = tex2D(_MainTex, i.uv + _MainTex_TexelSize.xy*float2( ix, iy)).rgb;
					float3 P2 = tex2D(_MainTex, i.uv + _MainTex_TexelSize.xy*float2(-ix,-iy)).rgb;
					float3 minP = P1;
					//Compute pixel value difference from center pixel
					float D1 = length(P1 - PCenter);
					float D2 = length(P2 - PCenter);
					//choose the pixel which is more similar to center pixel
					if (D2 < D1) minP = P2;
					AVG += minP;
				  }
				  AVG *= PcountInv;//take the average of the more similar pixel values
				  			
				return float4(AVG, 1);
			}
			ENDCG
		}
	}
}
